import json
import os

from catacomb import settings
from catacomb.common import constants, errors
from catacomb.utils import file_handler, formatter, helpers


def is_existing_tomb(ctx, tomb_name):
    """Checks if the tomb, specified by the given name, exists in the catacomb.

    Arguments:
        tomb_name (str): The name of the tomb.

    Returns:
        A `bool`, True if the specified name corresponds to an existing tomb,
        False otherwise.
    """
    return os.path.isfile(os.path.join(ctx.obj.catacomb_dir, tomb_name))


def get_current_tomb_name(ctx):
    """Retrieves the name of the current tomb.

    Returns:
        The name of the current tomb as a `string`.
    """
    return ctx.obj.open_tomb_name


def catacomb_to_table(ctx):
    """Converts the catacomb in to a table containing information about each
    tomb stored.

    Returns:
        A `string` representation of the table.
    """
    # Collect all the available tomb paths.
    for root, dirnames, names in os.walk(ctx.obj.catacomb_dir):
        tomb_names = sorted(names)
        tomb_paths = [os.path.join(root, name) for name in tomb_names]

    # Open each tomb and grab the description.
    tombs = []
    for idx, path in enumerate(tomb_paths):
        tomb = file_handler.read(path)
        tombs.append((tomb_names[idx], tomb["description"]))

    # Construct a table row for each tomb.
    rows = []
    for tomb in tombs:
        name, desc = tomb
        rows.append(formatter.create_row(name, desc))

    if len(rows):
        return formatter.to_table(constants.TABLE_HEADERS_TOMB, rows)

    return None


def create_tomb(ctx, tomb_name, description):
    """Creates a new tomb with the provided tomb name.

    Arguments:
        tomb_name (str): The name of the new tomb.
    """
    new_tomb_path = os.path.join(ctx.obj.catacomb_dir, tomb_name)

    tomb_contents = settings.DEFAULT_TOMB_CONTENTS
    tomb_contents["description"] = description

    file_handler.create(new_tomb_path, tomb_contents)


def open_tomb(ctx, tomb_name):
    """Opens the specified tomb, granting access to its contents/commands to
    the user.

    Arguments:
        tomb_name (str): The name of the new tomb.
    """
    if is_existing_tomb(ctx, tomb_name):
        ctx.obj.set_open_tomb(tomb_name)
    else:
        helpers.exit(errors.TOMB_OPEN_UNKNOWN.format(tomb_name))


def remove_tomb(ctx, tomb_name):
    """Removes a tomb from the catacomb.

    Arguments:
        tomb_name (str): The name of the new tomb.
    """
    if is_existing_tomb(ctx, tomb_name):
        os.remove(os.path.join(ctx.obj.catacomb_dir, tomb_name))
    else:
        helpers.exit(errors.TOMB_BURY_UNKNOWN.format(tomb_name))



# General application errors.
INVALID_COLOR = "Error 100: The desired color is not available, '{0}'."
INVALID_FORMAT_USE_CMD = (
    "Error 101: A formatting error occured and we're not sure why. Please "
    "report this at https://github.com/mitchjeitani/catacomb.\nCommand used "
    "was: {0}.")

# Catacomb handling errors.
TOMB_OPEN_UNKNOWN = (
    "Error 200: Tried to open a tomb that does not exist, '{0}'.")
TOMB_BURY_UNKNOWN = (
    "Error 201: Tried to bury a tomb that does not exist, '{0}'.")

# File handling errors.
FILE_CREATE_OVERWRITE = (
    "Error 300: Tried to overwrite a file by creating a new one at '{0}'.")
FILE_READ_UNKNOWN = (
    "Error 301: Tried to read a file that doesn't exist at '{0}'.")
FILE_UPDATE_UNKNOWN = (
    "Error 302: Tried to update a file that doesn't exist at {0}.")



import click

from catacomb.common import constants
from catacomb.utils import formatter, catacomb_handler


@click.command(
    constants.CMD_LIST_CATACOMB_NAME, help=constants.CMD_LIST_CATACOMB_DESC,
    short_help=constants.CMD_LIST_CATACOMB_DESC)
@click.pass_context
def list(ctx):
    """Lists all the tombs in the catacomb.
    """
    table = catacomb_handler.catacomb_to_table(ctx)
    if table is None:
        formatter.print_warning(constants.WARN_EMPTY_CATACOMB)
    else:
        click.echo(table)



import click

from catacomb.common import constants
from catacomb.utils import formatter, helpers, tomb_handler


@click.command(
    constants.CMD_ADD_NAME, help=constants.CMD_ADD_DESC,
    short_help=constants.CMD_ADD_DESC)
@click.argument("alias", nargs=1)
@click.argument("command", nargs=1, required=False)
@click.pass_context
def add(ctx, alias, command):
    """Adds a new command to the tomb.

    First prompting the user to specify the desired alias as well as a
    description of what the command does.

    Arguments:
        alias (str): The alias of the command being added to the tomb.
        command (str): The command to add to the tomb.
    """
    if tomb_handler.is_existing_command(ctx, alias):
        # If the `alias` specified is already used for a command in the tomb,
        # prompt the user to determine if they want to overwrite the old
        # command.
        update = click.prompt(constants.CMD_ADD_UPDATE_PROMPT.format(alias))
        if update.lower() != "y":
            # Abort the action.
            helpers.exit(constants.WARN_ACTION_ABORTED)

    if not command:
        # If the `command` hasn't yet been specified we'll need to prompt the
        # user for that as well.
        command = click.prompt(constants.CMD_ADD_CMD_PROMPT)

    description = click.prompt(constants.CMD_ADD_DESC_PROMPT)

    # Save the new command to the current tomb.
    tomb_handler.add_command(ctx, command, alias, description)
    formatter.print_success(constants.CMD_ADD_OK.format(alias, description))



import json
import os

from catacomb.common import constants, errors
from catacomb.utils import helpers


def create(path, contents=None):
    """Creates a new file at the given path.

    Arguments:
        contents (str): The file contents.
    """
    if os.path.exists(path):
        helpers.exit(errors.FILE_CREATE_OVERWRITE.format(path))

    if not contents:
        # Allow the creation of an empty file.
        contents = ""

    with open(path, "w") as f:
        f.write(json.dumps(contents, indent=constants.INDENT_NUM_SPACES))


def update(path, contents):
    """Updates the file at the given path with the provided contents.

    Arguments:
        contents (str): The file contents.
    """
    if not os.path.exists(path):
        helpers.exit(errors.FILE_UPDATE_UNKNOWN.format(path))

    with open(path, "w") as f:
        f.write(json.dumps(contents, indent=constants.INDENT_NUM_SPACES))


def read(path):
    """Read the file at the given path.

    Arguments:
        path (str): The path of the file to read.

    Returns:
        A `dict`, representing the contents of the file, if the file exists.
    """
    if not os.path.exists(path):
        helpers.exit(errors.FILE_READ_UNKNOWN.format(path))

    with open(path, "r") as f:
        contents = json.load(f)

    return contents



import click
import os

from catacomb.common import constants, errors
from catacomb.utils import formatter, helpers, tomb_handler


@click.command(
    constants.CMD_USE_NAME, help=constants.CMD_USE_DESC,
    short_help=constants.CMD_USE_DESC)
@click.argument("alias", nargs=1)
@click.argument("params", nargs=-1)
@click.pass_context
def use(ctx, alias, params):
    """Retrieves a command from the tomb and executes it.

    Arguments:
        alias (str): The alias of the command to execute.
        params (tuple): Optional parameters to be formatted into the command.
    """
    cmd = tomb_handler.get_command(ctx, alias)

    if cmd:
        if params:
            # Substitute any placeholders in the command with the provided
            # parameters.
            cmd = format_cmd(alias, cmd, params)
        # Execute the command.
        os.system(cmd)
    else:
        # The command alias doesn't exist.
        formatter.print_warning(constants.WARN_CMD_NOT_FOUND.format(alias))


def format_cmd(alias, cmd, params):
    """Formats a command with user provided parameters, similar to the Python
    `format()` method.

    Arguments:
        cmd (str): The command.
        params (tuple): Parameters to be formatted into the command.

    Returns:
        A `string` representing the newly formatted command.
    """
    try:
        # In the case of more parameters vs placeholders, we will exhaust what
        # we can (in order) and run the command anyway. If nothing can be
        # substituted, the formatting will be ignored and the command will be
        # run as is.
        return cmd.format(*params)
    except IndexError:
        # Not all the placeholders are provided with a value.
        helpers.exit(constants.WARN_FMT_NUM_PARAMS.format(
            alias, len(params)))
    except KeyError:
        # Placeholders aren't following the correct syntax.
        helpers.exit(constants.WARN_FMT_PLACEHOLDER_SYNTAX)
    except ValueError:
        # Inconsistent use of placeholders in a command.
        helpers.exit(constants.WARN_FMT_PLACEHOLDER_SYNTAX2)
    except Exception:
        # This shouldn't be reachable, but just in case, we'll report it.
        helpers.exit(errors.INVALID_FORMAT_USE_CMD.format(cmd))



import click
import os

from catacomb.common import constants
from catacomb.utils import formatter, helpers, tomb_handler

from subprocess import call


@click.command(
    constants.CMD_EDIT_NAME, help=constants.CMD_EDIT_DESC,
    short_help=constants.CMD_EDIT_DESC)
@click.argument("alias", nargs=1)
@click.pass_context
def edit(ctx, alias):
    """Edits a command if it's present in the active tomb.

    Opens an editor for the user with the details for the specified command,
    allowing edits to the alias, command and description.

    Arguments:
        alias (str): The alias of the command to edit.
    """
    if not tomb_handler.is_existing_command(ctx, alias):
        helpers.exit(constants.WARN_CMD_NOT_FOUND.format(alias))

    editor = os.environ.get("EDITOR", helpers.default_editor())
    cmd_info = tomb_handler.get_command(ctx, alias, True)
    fname = helpers.random_fname()

    # Creates a temporary file with the current command information, allowing
    # edits.
    with open(fname, "w") as temp_file:
        # Append the initial message to the file.
        temp_file.write(constants.CMD_EDIT_INIT_MSG.format(
            alias, cmd_info[0], cmd_info[1]))

    # Open the file within an editor.
    call([editor, temp_file.name])

    # Reads the changes to the file after the user is done with it.
    with open(fname, "r") as temp_file:
        edits = read_edits(temp_file)

    # Clean up.
    os.remove(fname)

    # The edited alias already exists in the active tomb.
    if edits["alias"] != alias and tomb_handler.is_existing_command(
            ctx, edits["alias"]):
        update = click.prompt(
            constants.CMD_EDIT_OVERWRITE_PROMPT.format(alias))

        if update.lower() != "y":
            # Abort the action.
            helpers.exit(constants.WARN_ACTION_ABORTED)
        else:
            # We need to remove the old command if we're not overwriting it.
            tomb_handler.remove_command(ctx, alias)
    elif not tomb_handler.is_existing_command(ctx, edits["alias"]):
        # Remember to remove the old command if we're adding a new one.
        tomb_handler.remove_command(ctx, alias)

    # Update the command.
    tomb_handler.add_command(
        ctx, edits["command"], edits["alias"], edits["description"])
    formatter.print_success(constants.CMD_EDIT_OK)


def read_edits(f):
    """Read the edits from the provided file.

    Arguments:
        f (file): The file, containing user edits.

    Returns:
        A `dict` containing the new alias, command and description.
    """
    d = dict()

    for line in f:
        if "=" in line:
            # Strip out the edited alias, command and description from the
            # file.
            entry = [ln.strip() for ln in line.split("=", 1)]
            d[entry[0].lower()] = entry[1]

    # Check if the edits are valid, if not, cancel execution and print the
    # proper error message.
    err_message = validate_edits(d)
    if err_message:
        helpers.exit(err_message)

    return d


def validate_edits(edits):
    """Check if the edits are valid.

    Arguments:
        edits (dict): Contains the edits to validate.

    Returns:
        A `str` describing the error, None if there are no errors.
    """
    # There should be an entry for the alias, command and description.
    num_entries = len(edits)
    if num_entries != 3:
        return constants.CMD_EDIT_MISSING_KEYS.format(num_entries)

    # Ensure that each key is present.
    if "alias" not in edits:
        return constants.CMD_EDIT_MISSING_KEY.format("alias")
    if "command" not in edits:
        return constants.CMD_EDIT_MISSING_KEY.format("command")
    if "description" not in edits:
        return constants.CMD_EDIT_MISSING_KEY.format("description")

    # Ensure that each key has a value.
    if not len(edits["alias"]):
        return constants.CMD_EDIT_MISSING_VAL.format("alias")
    if not len(edits["command"]):
        return constants.CMD_EDIT_MISSING_VAL.format("command")
    if not len(edits["description"]):
        return constants.CMD_EDIT_MISSING_VAL.format("description")

    return None



import json
import os

from catacomb import settings
from catacomb.common import constants
from catacomb.utils import file_handler


class Context(object):
    """Custom context that is passed through the use of the `@pass_context`
    decorator. Contains important configuration that needs to be accessible
    throughout the application.

    Attributes:
        config_dir (str): The path to catacombs local configuration and
            storage directories.
        config_file_path (str): Path to the local configuration file.
        catacomb_dir (str): Path to the catacomb directory that stores
            the contents of every tomb.
        open_tomb_name (str): The name of the currently open tomb.
        open_tomb_dir (str): The path to the currently open tomb.
    """

    def __init__(self):
        self.config_dir = os.path.join(
            os.path.expanduser("~"), settings.CONFIG_DIR_NAME)
        self.config_file_path = os.path.join(
            str(self.config_dir), settings.CONFIG_FILE_NAME)
        self.catacomb_dir = os.path.join(
            self.config_dir, settings.TOMB_DIR_NAME)
        self.open_tomb_name = settings.TOMB_DEFAULT_FILE_NAME
        self.open_tomb_dir = os.path.join(
            self.catacomb_dir, self.open_tomb_name)
        self._init_catacomb()

    def _init_catacomb(self):
        """Initialises the users configuration for the application.
        """
        # Initialise various directories required for the application if they
        # don't already exist.
        if not os.path.exists(self.config_dir):
            os.makedirs(self.config_dir)

        if not os.path.exists(self.catacomb_dir):
            os.makedirs(self.catacomb_dir)

        # Initialise user configuration and other defaults if they don't
        # already exist.
        if not os.path.isfile(self.config_file_path):
            file_handler.create(self.config_file_path, settings.DEFAULT_CONFIG)
        else:
            # If configuration does exist, read which tomb we should be using.
            config = file_handler.read(self.config_file_path)
            self.open_tomb_name = config["open_tomb_name"]
            self.open_tomb_dir = os.path.join(
                self.catacomb_dir, self.open_tomb_name)

        # Initialise a default tomb if no tomb currently exists.
        if not os.path.isfile(self.open_tomb_dir):
            file_handler.create(
                self.open_tomb_dir, settings.DEFAULT_TOMB_CONTENTS)

    def set_open_tomb(self, tomb_name):
        """Sets the current open tomb path to that of the specified tomb.

        Arguments:
            tomb_name (str): The name of the tomb.
        """
        new_tomb_path = os.path.join(self.catacomb_dir, tomb_name)

        if os.path.isfile(new_tomb_path):
            # Update the name of the open tomb and write it to the config file.
            config = file_handler.read(self.config_file_path)
            config["open_tomb_name"] = tomb_name
            file_handler.update(self.config_file_path, config)
            self.open_tomb_dir = new_tomb_path



# Configuration naming.
CONFIG_DIR_NAME = ".catacomb"
CONFIG_FILE_NAME = ".catacomb_config"

TOMB_DIR_NAME = "tombs"
TOMB_DEFAULT_FILE_NAME = "default"

# Default user configuration.
DEFAULT_CONFIG = {
    "open_tomb_name": TOMB_DEFAULT_FILE_NAME
}

DEFAULT_EDITOR_UNIX = "vi"
DEFAULT_EDITOR_WIN32 = "edit"
DEFAULT_EDITOR_WIN64 = "notepad"

DEFAULT_TOMB_CONTENTS = {
    "commands": {},
    "description": "-"
}

# Settings to override on the context object.
CONTEXT_SETTINGS = {
    "help_option_names": ["-h", "--help"]
}



import click

from catacomb.common import constants
from catacomb.utils import catacomb_handler, formatter


@click.command(
    constants.CMD_BURY_NAME, help=constants.CMD_BURY_DESC,
    short_help=constants.CMD_BURY_DESC)
@click.argument("tomb_name", nargs=1)
@click.option(
    "--force", "-f", is_flag=True, default=False,
    help=constants.CMD_BURY_FORCE_DESC)
@click.pass_context
def bury(ctx, tomb_name, force):
    """Removes the specified tomb from the catacomb.

    Arguments:
        tomb_name (str): The name/alias of the tomb.
        force (bool): Option to disable prompting the user for confirmation.
    """
    if catacomb_handler.get_current_tomb_name(ctx).lower() == tomb_name:
        # Can't bury a tomb if it's currently being used.
        formatter.print_warning(constants.CMD_BURY_SELF_WARN.format(tomb_name))
    elif not catacomb_handler.is_existing_tomb(ctx, tomb_name):
        # If the tomb, specified by the user, can not be found, do nothing.
        formatter.print_warning(
            constants.WARN_TOMB_NOT_FOUND.format(tomb_name))
    else:
        if not force:
            # Prompt the user for confirmation on burying the tomb.
            confirm = click.prompt(constants.CMD_BURY_PROMPT.format(tomb_name))

        if force or confirm.lower() == "y":
            catacomb_handler.remove_tomb(ctx, tomb_name)
            formatter.print_success(constants.CMD_BURY_OK.format(tomb_name))
        else:
            formatter.print_warning(constants.WARN_ACTION_ABORTED)



# Entry points.
DESC_CATACOMB = (
    "For handling the tombs within your catacomb. Add the flags -h or --help "
    "to any of the commands for more information, e.g. catacomb COMMAND -h.")
DESC_TOMB = (
    "For handling the contents within a tomb. Add the flags -h or --help "
    "to any of the commands for more information, e.g. tomb COMMAND -h.")

# Misc values.
INDENT_NUM_SPACES = 2

# Table properties.
TABLE_HEADERS_CMD = ["Alias", "Command", "Description"]
TABLE_HEADERS_TOMB = ["Name", "Description"]
TABLE_COL_MAX_WIDTH = 30

# `catacomb` subcommand definitions.
CMD_BURY_NAME = "bury"
CMD_BURY_DESC = "Permanently buries a tomb, preventing any further use."
CMD_BURY_PROMPT = (
    "You're about to bury the tomb '{0}' for good. Would you like to continue"
    "? (Y/n)")
CMD_BURY_FORCE_DESC = "Ignore the prompt for user confirmation."
CMD_BURY_OK = ("The tomb '{0}' has been removed from the catacomb.")
CMD_BURY_SELF_WARN = (
    "The tomb '{0}' is currently open and can not be buried.")

CMD_CREATE_NAME = "create"
CMD_CREATE_DESC = "Creates a new tomb in the catacomb."
CMD_CREATE_DESC_PROMPT = "Description"
CMD_CREATE_FORCE_DESC = "Overwrite a tomb if it already exists."
CMD_CREATE_OK = "A new tomb has been constructed with alias '{0}'."

CMD_LIST_CATACOMB_NAME = "list"
CMD_LIST_CATACOMB_DESC = "Lists the tombs currently available in the catacomb."

CMD_OPEN_NAME = "open"
CMD_OPEN_DESC = "Opens an existing tomb."
CMD_OPEN_OK = "Switched to tomb '{0}'."
CMD_OPEN_NEW_DESC = "Creates and opens a new tomb."
CMD_OPEN_NEW_DESC_PROMPT = "Description"
CMD_OPEN_NEW_OK = "Switched to new tomb '{0}'."
CMD_OPEN_SELF_WARN = "Tomb '{0}' is already open."

# `tomb` subcommand definitions.
CMD_ADD_NAME = "add"
CMD_ADD_DESC = "Stores a new command in the tomb."
CMD_ADD_OK = (
    "Successfully added the command with alias '{0}', and "
    "description '{1}' to the tomb.")
CMD_ADD_CMD_PROMPT = "Command"
CMD_ADD_DESC_PROMPT = "Description"
CMD_ADD_UPDATE_PROMPT = (
    "The alias '{0}' is already being used in this tomb. Would "
    "you like to overwrite it? (Y/n)")

CMD_CLEAN_NAME = "clean"
CMD_CLEAN_DESC = "Obliterates the contents of the active tomb."
CMD_CLEAN_OK = "The contents of the tomb have been cleared."
CMD_CLEAN_FORCE_DESC = "Ignore the prompt for user confirmation."
CMD_CLEAN_PROMPT = (
    "You're about to completely destroy the contents of this "
    "tomb. Would you like to continue? (Y/n)")

CMD_EDIT_NAME = "edit"
CMD_EDIT_DESC = (
    "Opens an editor to edit a command that's already stored in "
    "the active tomb.")
CMD_EDIT_OK = "The command was successfully updated."
CMD_EDIT_OVERWRITE_PROMPT = (
    "The alias '{0}' is already being used in this tomb. Would "
    "you like to overwrite it? (Y/n)")
CMD_EDIT_INIT_MSG = (
    "# You are currently editing '{0}'. Only edit text on the right hand\n"
    "# side of the equals sign, and keep entries on the same line.\n\n"
    "ALIAS = {0}\n\n"
    "COMMAND = {1}\n\n"
    "DESCRIPTION = {2}\n\n"
    "# End of file.\n")
CMD_EDIT_MISSING_KEY = (
    "The entry for '{0}' seems to have been corrupted. Don't remove any lines "
    "and only edit text on the right hand side of the equals sign. Please try "
    "again.")
CMD_EDIT_MISSING_KEYS = (
    "Oops, we found an unexpected number of entries. 3 expected, {0} were "
    "found. Please try again.")
CMD_EDIT_MISSING_VAL = ("A value for '{0}' was not entered, please try again.")

CMD_STATUS_NAME = "status"
CMD_STATUS_DESC = "Shows some general information about the active tomb."
CMD_STATUS_OK = (
    "You're currently using tomb '{0}', which contains {1} commands.")

CMD_USE_NAME = "use"
CMD_USE_DESC = "Executes a command, specified by its alias."

CMD_LIST_NAME = "list"
CMD_LIST_DESC = "Displays all the commands available in the active tomb."

CMD_RM_NAME = "rm"
CMD_RM_DESC = "Removes a command from the tomb."
CMD_RM_OK = "Successfully removed the command with alias '{0}' from the tomb."

# General command related warnings.
WARN_ACTION_ABORTED = "The action was aborted."
WARN_CMD_NOT_FOUND = (
    "The alias '{0}' doesn't correspond to any of the commands in this tomb.")
WARN_EMPTY_CATACOMB = "Nothing but empty rooms and dirt..."
WARN_EMPTY_TOMB = "Nothing but crumbled bones and dust..."
WARN_TOMB_EXISTS = "The tomb with alias '{0}' already exists."
WARN_TOMB_NOT_FOUND = (
    "The alias '{0}' doesn't correspond to any of the tombs in the catacomb.")
WARN_FMT_NUM_PARAMS = (
    "Not enough parameters to use '{0}', {1} were provided.")
WARN_FMT_PLACEHOLDER_SYNTAX = (
    "The placeholders in this command don't adhere to the correct syntax. "
    "Please either use empty curly brackets '{}' or specify the index with "
    "a number, e.g. '{0} {1} {2}'.")
WARN_FMT_PLACEHOLDER_SYNTAX2 = (
    "There are both types of placeholders in this command, '{}' and "
    "'{[0-9]+}'. Commands should use only one type.")



import click

from catacomb.common import constants
from catacomb.utils import catacomb_handler, formatter


@click.command(
    constants.CMD_CREATE_NAME, help=constants.CMD_CREATE_DESC,
    short_help=constants.CMD_CREATE_DESC)
@click.argument("tomb_name", nargs=1)
@click.option(
    "--force", "-f", is_flag=True, default=False,
    help=constants.CMD_CREATE_FORCE_DESC)
@click.pass_context
def create(ctx, tomb_name, force):
    """Creates a new tomb.

    Arguments:
        tomb_name (str): The name/alias of the tomb.
        force (bool): Option to disable prompting the user for confirmation.
    """
    # Check if there currently exists a tomb that has the same name as the
    # one specified before creating a new one.
    if not catacomb_handler.is_existing_tomb(ctx, tomb_name) or force:
        description = click.prompt(constants.CMD_CREATE_DESC_PROMPT)
        catacomb_handler.create_tomb(ctx, tomb_name, description)
        formatter.print_success(constants.CMD_CREATE_OK.format(tomb_name))
    else:
        formatter.print_warning(constants.WARN_TOMB_EXISTS.format(tomb_name))



import click

from catacomb import settings
from catacomb.common import constants
from catacomb.decorators.context import Context
from catacomb.utils.plugin_loader import PluginLoader

pass_context = click.make_pass_decorator(Context, ensure=True)


@click.group(
    context_settings=settings.CONTEXT_SETTINGS, help=constants.DESC_CATACOMB,
    cls=PluginLoader)
@pass_context
def catacomb_entry(ctx):
    """Entry point for the `catacomb` command. Passes responsibility of
    handling a command down the chain of subcommand handlers until we've found
    one that can handle the given subcommand.

    This entry point handles actions involving the operations that alter the
    state of the catacomb and tombs within it.
    """
    pass


@click.group(
    context_settings=settings.CONTEXT_SETTINGS, help=constants.DESC_TOMB,
    cls=PluginLoader)
@pass_context
def tomb_entry(ctx):
    """Entry point for the `tomb` command. Passes responsibility of handling
    a command down the chain of subcommand handlers until we've found one that
    can handle the given subcommand.

    This entry point handles actions involving the contents of the current
    tomb.
    """
    pass



import os
import platform
import sys
import uuid

from catacomb import settings
from catacomb.utils import formatter

from collections import defaultdict


def get_platform():
    """Gets the users operating system.

    Returns:
        An `int` representing the users operating system.
            0: Windows x86 (32 bit)
            1: Windows x64 (64 bit)
            2: Mac OS
            3: Linux
        If the operating system is unknown, -1 will be returned.
    """
    return defaultdict(lambda: -1, {
        "Windows": 1 if platform.machine().endswith("64") else 0,
        "Darwin": 2,
        "Linux:": 3,
    })[platform.system()]


def default_editor():
    """Retrieves the platforms respective 'default' editor.

    Returns:
        A `str` representing the editor.
    """
    return defaultdict(lambda: settings.DEFAULT_EDITOR_UNIX, {
        0: settings.DEFAULT_EDITOR_WIN32,
        1: settings.DEFAULT_EDITOR_WIN64,
    })[get_platform()]


def random_fname():
    """Generates a random file name. In the *very* unlikely case that `uuid4`
    generates a file name that already exists, we'll generate a new one until
    a unique file name is generated.

    Returns:
        A unique file name `str`.
    """
    fname = "{}.tmp".format(str(uuid.uuid4()))

    while (os.path.isfile(fname)):
        fname = "{}.tmp".format(str(uuid.uuid4()))

    return fname


def exit(message):
    """Prints an error and forces execution of the application to stop.

    Arguments:
        message (str): An error message.
    """
    formatter.print_error(message)
    sys.exit(1)



import json

from catacomb.common import constants
from catacomb.utils import file_handler, formatter


def read_tomb_commands(ctx):
    """Gets the commands currently present in the tomb.

    Returns:
        A `dict` representing the commands in the tomb.
    """
    tomb_data = file_handler.read(ctx.obj.open_tomb_dir)
    return tomb_data["commands"]


def update_tomb_commands(ctx, cmds):
    """Replaces the current commands in the tomb with `cmds`.

    Arguments:
        cmds (dict): The commands to store in the tomb.
    """
    # We need to preserve the tombs other attributes (e.g. description), and
    # only need to write to "commands".
    tomb_contents = file_handler.read(ctx.obj.open_tomb_dir)
    tomb_contents["commands"] = cmds

    file_handler.update(ctx.obj.open_tomb_dir, tomb_contents)


def clean_tomb(ctx):
    """Clears the commands currently present in the tomb, resetting it to it's
    original (empty) state.
    """
    update_tomb_commands(ctx, {})


def is_existing_command(ctx, alias):
    """Checks if a command belonging to the given alias exists in the active
    tomb.

    Arguments:
        alias (str): The command alias.

    Returns:
        A `bool`.
    """
    data = read_tomb_commands(ctx)
    return alias in data


def add_command(ctx, command, alias, description):
    """Adds a new command to the current tomb.

    Arguments:
        command (str): The command to add.
        alias (str): The alias to save the command as.
        description (str): What the command does.
    """
    data = read_tomb_commands(ctx)

    data[alias] = {
        "command": command,
        "description": description
    }

    update_tomb_commands(ctx, data)


def get_command(ctx, alias, desc=False):
    """Retrieves a command from the current tomb, using its alias.

    Arguments:
        alias (str): The alias to save the command as.
        desc (bool): If True, return the description as well (default: False).

    Returns:
        The command as a `string`, or a `tuple` containing the command and
        description if desc is True, or None if not found.
    """
    data = read_tomb_commands(ctx)

    if alias in data:
        if desc:
            return (data[alias]["command"], data[alias]["description"])
        return data[alias]["command"]

    return None


def remove_command(ctx, alias):
    """Removes a command from the current tomb.

    Arguments:
        alias (str): The alias to save the command as.

    Returns:
        A `bool`, True if the alias could be removed, False otherwise.
    """
    data = read_tomb_commands(ctx)

    if alias not in data:
        return False

    # Remove the command then write back to the file.
    del data[alias]
    update_tomb_commands(ctx, data)

    return True


def get_num_commands(ctx):
    """Counts the number of commands present in the current tomb.

    Returns:
        An `int`, representing the number of commands present in the tomb.
    """
    return len(read_tomb_commands(ctx))


def tomb_to_table(ctx):
    """Converts the current tomb to a table containing information about each
    command stored.

    Returns:
        A `string` representation of the table.
    """
    data = read_tomb_commands(ctx)

    # Convert each stored command to it's own row.
    rows = []
    for alias in data.keys():
        cmd = data[alias]["command"]
        desc = data[alias]["description"]
        rows.append(formatter.create_row(alias, cmd, desc))

    if len(rows):
        return formatter.to_table(constants.TABLE_HEADERS_CMD, rows)

    return None



import click

from catacomb.common import constants
from catacomb.utils import catacomb_handler, formatter


@click.command(
    constants.CMD_OPEN_NAME, help=constants.CMD_OPEN_DESC,
    short_help=constants.CMD_OPEN_DESC)
@click.argument("tomb_name", nargs=1)
@click.option(
    "--new", "-n", is_flag=True, default=False,
    help=constants.CMD_OPEN_NEW_DESC)
@click.pass_context
def open(ctx, tomb_name, new):
    """Opens the tomb specified by the user, granting access to all the
    commands stored within it.

    Arguments:
        tomb_name (str): The name/alias of the tomb.
        new (bool): If True, create a new tomb (if it doesn't exist) and then
            switch to it.
    """
    if catacomb_handler.get_current_tomb_name(ctx).lower() == tomb_name:
        # Don't do anything if the specified tomb is already open.
        if new:
            formatter.print_warning(constants.WARN_TOMB_EXISTS.format(
                tomb_name))
        else:
            formatter.print_warning(constants.CMD_OPEN_SELF_WARN.format(
                tomb_name))
    elif new:
        # Create a new tomb and switch to it.
        if not catacomb_handler.is_existing_tomb(ctx, tomb_name):
            description = click.prompt(constants.CMD_OPEN_NEW_DESC_PROMPT)
            catacomb_handler.create_tomb(ctx, tomb_name, description)
            catacomb_handler.open_tomb(ctx, tomb_name)
            formatter.print_success(constants.CMD_OPEN_NEW_OK.format(
                tomb_name))
        else:
            # Do nothing if a tomb with the provided alias already exists.
            formatter.print_warning(constants.WARN_TOMB_EXISTS.format(
                tomb_name))
    elif catacomb_handler.is_existing_tomb(ctx, tomb_name):
        # Otherwise open a new tomb if it exists.
        catacomb_handler.open_tomb(ctx, tomb_name)
        formatter.print_success(constants.CMD_OPEN_OK.format(tomb_name))
    else:
        formatter.print_warning(constants.WARN_TOMB_NOT_FOUND.format(
            tomb_name))



import click

from catacomb.common import constants
from catacomb.utils import catacomb_handler, formatter, tomb_handler


@click.command(
    constants.CMD_STATUS_NAME, help=constants.CMD_STATUS_DESC,
    short_help=constants.CMD_STATUS_DESC)
@click.pass_context
def status(ctx):
    """Displays the status of the current tomb.
    """
    current_tomb_name = catacomb_handler.get_current_tomb_name(ctx)
    num_commands = tomb_handler.get_num_commands(ctx)

    click.echo(constants.CMD_STATUS_OK.format(current_tomb_name, num_commands))
